/*
###########################################################
PATTERN GENERTOR 
	for AT90S8535 @ 8Mhz
-----------------------------------------------------------
Features:
	Two Input Modes:
	1.	Manual Data Input
		- 100 data sets possible
		- binary input 
		
	2.	Counter
		- Up/Down
		- Individual Start/End value, step width
	
	Output:
		- individual delay
		
	Saving Data to EEprom
	
-----------------------------------------------------------
Version: 		Beta 1
Built:			10/26/2004
(c) by Martin Jansen
-----------------------------------------------------------
Flowchart:		pattern generator.flo
Schematics:		bitmuster.sch
-----------------------------------------------------------
-----------------------------------------------------------
Pin-Connnections:
	LCD			PortC
	Buttons		PortB(0,1)			low-active!
	74hc744 enable PortB2			low-active
	DataIn		PortD
	OUtput		PortA				74hc244-buffer
-----------------------------------------------------------
-----------------------------------------------------------
EEprom:
	0x00		mode
	0x01		out_delay
	0x02		n_data	/	step_width
	0x03		data0	/	start_value
	0x04		data1	/	end_value
	0x05		data2
	0x06		
	....
	0x66		data99
	
###########################################################
*/

#include <stdlib.h>
#include <avr/io.h>
#include <inttypes.h>
#include <avr/eeprom.h>
#include "lcd.h"
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

#define max_data 100				//Maximum Number of Data Sets
#define b0 PINB0					//Buttons
#define b1 PINB1
#define mode flags.f_mode			//for better understanding...
#define op flags.f_op
#define firstloop flags.f_loop
#define allow_cancel flags2.f_cancel
#define canceled flags2.f_last

//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
int max = max_data;
int data[max_data];
int n_data;							//number of data sets
int new_data;
int old_data;
int start_value;					//Countermode
int end_value;						
int step_width;
int i;								//counts: output loops, during load/save loops eeprom
long t;								//counter for delays, may be used universally 
char buffer[7];					//for conversion (lcd prompt)
int out_delay;						//output delay: user enters a binary number (0-255), which
									//is internally multiplied with 10000 by combining 2 for-loops
int t_out_delay;					//counter for output-delay loop
int code;							//op-election: code
struct
{
	unsigned char f_mode:3; 		//mode={0:reset; 1:Data; 2:Counter; 3:Load}
	unsigned char f_op:3;			//op {0:output; 1:save; 2:reconfigure delay; 3:reconfigure data; 4:reset}
	unsigned char f_loop:1;		//indicates first loop during input
} flags;
struct
{
	unsigned char f_cancel:1;		//user may cancel input
	unsigned char f_last:1;		//user canceled
} flags2;


//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

//Functions
int init(void)						
	//initializing after reset
{
	DDRA=0xFF;						//A: Out (Data)
	DDRC=0xFF;						//C: Out (LCD)
	DDRB=0x04;						//B: Out - 2x In (Buttons)
	DDRD=0;							//D: In (DIP)
	PORTB=0x03;						//Pullups B (buttons-->Gnd)
	PORTD=0xFF;						//Pullups D (DIP-->Gnd)
	lcd_init(LCD_DISP_ON);
	sbi (PORTB,2);					//disable output
}

int bin_out(int number)
	//at current cursorposition
{
	if (number<128) lcd_puts("0");
	if (number<64) lcd_puts("0");
	if (number<32) lcd_puts("0");
	if (number<16) lcd_puts("0");
	if (number<8) lcd_puts("0");
	if (number<4) lcd_puts("0");
	if (number<2) lcd_puts("0");
	lcd_puts(itoa(number,buffer,2));	
}

int set_mode(void)
{
	lcd_clrscr();
	lcd_puts("Mode:");
	lcd_gotoxy(0,1);
	lcd_puts("Switch: B1");

	for(mode=1;mode<=4;mode++)
	{
		if (mode==4) mode=1;				//user can't chose "reset"
		lcd_gotoxy(0,3);
		lcd_puts(itoa(mode,buffer,10));	//prints mode
		if (mode==1) lcd_puts(" Data In");
		if (mode==2) lcd_puts(" Counter");
		if (mode==3) lcd_puts(" Load   ");
		while(1)
		{
			if (bit_is_clear(PINB,b0)) 			//pressing b0 affirmatives election
			{
				press_key(0);
				return mode;
			}
			if (bit_is_clear(PINB,b1)) 			//pressing b1 switches to next mode#
			{
				press_key(1); 
				break;
			}
		}						
	}		

	
}
int press_key(int key)
	//waits until button is pushed and released
{
	loop_until_bit_is_clear(PINB,key);			
	for(t=0;t<=500000;t++)						//debouncing
	{
		asm volatile ("nop");
	}
	loop_until_bit_is_set(PINB,key);
}
int pattern_input(void)
{
	allow_cancel=1;									//user may cancel input
	lcd_clrscr();
	for(n_data=1;n_data<=max;n_data++)
	{
		lcd_gotoxy(0,0);							//number of pattern:
		lcd_puts(itoa(n_data,buffer,10));			//	n_data/max_data
		lcd_puts("/");
		lcd_puts(itoa(max,buffer,10));
		data[(n_data-1)]=data_in();
		if (canceled==1)							//user canceled input
		{
		canceled=0;
		break;
		}
	}
	allow_cancel=0;
}


int data_in(void)
{
	firstloop=1;
	lcd_gotoxy(0,3);
	lcd_puts("B0:Save");
	if (allow_cancel==1) lcd_puts("  B1:End");
	while(1)
	{
		new_data=PIND;
		if ((new_data!=old_data) || (firstloop==1))	//Prints data only if data is new
		{
			firstloop=0;
			old_data=new_data;
			//bin out:
				lcd_gotoxy(0,1);
				lcd_puts("        ");
				lcd_gotoxy(0,1);
				bin_out(new_data);
				lcd_puts("  ");
				if (new_data<100) lcd_puts(" ");
				if (new_data<10) lcd_puts(" ");
				lcd_puts(itoa(new_data,buffer,10));
		}
		if (bit_is_clear(PINB,b0)) 			//pressing b0 saves value
		{
			press_key(0); 
			break;
		}
		if ((bit_is_clear(PINB,b1)) && (allow_cancel==1))	//pressing b1 cancels input, but last value is saved!
		{
			press_key(1);
			canceled=1;
			break;
		}
	}
	return new_data;
}

int counter_setup(void)
{
	allow_cancel=0;
	lcd_clrscr();
	lcd_puts("Start");
	start_value=data_in();
	lcd_clrscr();
	lcd_puts("End");
	end_value=data_in();
	lcd_clrscr();
	lcd_puts("Step");
	step_width=data_in();
}

int set_delay(void)
{
	lcd_clrscr();
	lcd_puts("Delay");
	lcd_gotoxy(4,2);
	lcd_puts("x10000");
	out_delay=data_in();
	if (out_delay==0) out_delay=1;
}
int delay(void)
	//delay during output
{
	for(t_out_delay=0;t_out_delay<=out_delay;t_out_delay++)
	{
		for(t=0;t<=10000;t++)
		{
			asm volatile ("nop");
		}
	}
}
int output(void)
{
	lcd_clrscr();
	lcd_puts("B0: Start");
	press_key(0);	//wait until b0 is pressed
	lcd_clrscr();
	lcd_puts("Output...");
	cbi (PORTB,2);	//enable output
	if (mode==1) //Datamode
	{
		for(i=0;i<=(n_data-1);i++)
		{
			PORTA=data[i];
			delay();
		}
	}
	if (mode==2) //counter
	{
		for(i=start_value;i<=end_value;i=(i+step_width))
		{
			PORTA=i;
			delay();
		}
	}
	sbi (PORTB,2);	//disable output
}
int chose_op(void)
{
	lcd_clrscr();
	lcd_puts("Chose Task");
	lcd_gotoxy(0,1);
	lcd_puts("B1: Switch");
	while(1)
	{
		for(code=0;code<=5;code++)
		{
			if (code==5) code=0;
			lcd_gotoxy(0,3);
			if (code==0) lcd_puts("Restart Output");
			if (code==1) lcd_puts("Save to EEprom");
			if (code==2) lcd_puts("Reset Delay   ");
			if (code==3) lcd_puts("Reset Data    ");
			if (code==4) lcd_puts("Reset All     ");
			while(1)
			{
				if (bit_is_clear(PINB,b0)) 			//pressing b0 affirmatives election
				{
					press_key(0);
					return code;
				}
				if (bit_is_clear(PINB,b1)) 			//pressing b1 switches to next mode#
				{
					press_key(1); 
					break;
				}
			}	
		}
	}
}
int load_data(void)
{
	mode=eeprom_rb(0);
	out_delay=eeprom_rb(1);
	if (mode==1)
	{
		n_data=eeprom_rb(2);
		for(i=0;i<=(n_data-1);i++)
		{
			data[i]=eeprom_rb(i+3);
		}
	}
	if (mode==2)
	{
		step_width=eeprom_rb(2);
		start_value=eeprom_rb(3);
		end_value=eeprom_rb(4);
	}
}
int save_data(void)
{
	eeprom_wb(0,mode);
	eeprom_wb(1,out_delay);
	if (mode==1)
	{
		eeprom_wb(2,n_data);
		for(i=0;i<=(n_data-1);i++)
		{
			eeprom_wb((i+3),data[i]);
		}
	}
	if (mode==2)
	{
		eeprom_wb(2,step_width);
		eeprom_wb(3,start_value);
		eeprom_wb(4,end_value);
	}
	op=4;			//reset
}
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

int main(void)
{
	init();
	mode=0;							//reset
	op=4;							//reset
	
	//....................................................
	
	while(1)						//PROGRAMMLOOP
	{
		if (mode==0) set_mode();	
		if (mode==3) load_data();
		if ((mode==1) && (n_data==0)) pattern_input();		//lets the user put in the patterns, n_data=0 indicates 0 patterns
		if ((mode==2) && (step_width==0)) counter_setup();	//step_width=0 -> counter isn't initialized
		if (((mode==1) || (mode==2)) && (out_delay==0)) set_delay();//delay=0 -> no delay
		if ((mode==1) && (out_delay!=0)) output();
		if ((mode==2) && (step_width!=0) && (out_delay!=0)) output();
		op=chose_op();				//"What next?"
		if (op==1) save_data();	//save, then reset
		if (op==2) out_delay=0;
		if ((op==3) && (mode==1)) n_data=0;
		if ((op==3) && (mode==2)) step_width=0;
		if (op==4) 
		{
			n_data=0;
			out_delay=0;
			step_width=0;
			mode=0;
		}
		
	}
	
}


